==> py_import_cycles-0.2.2.tar.gz: py_import_cycles-0.2.2/py_import_cycles/tarjan.py <==
#!/usr/bin/env python3

from collections.abc import Hashable
from typing import List, Mapping, MutableMapping, Sequence, Tuple, TypeVar

T = TypeVar("T", bound=Hashable)


def strongly_connected_components(
    graph: Mapping[T, Sequence[T]],
) -> Sequence[Tuple[T, ...]]:
    """
    Tarjan's Algorithm (named for its discoverer, Robert Tarjan) is a graph theory algorithm
    for finding the strongly connected components of a graph.

    Based on: http://en.wikipedia.org/wiki/Tarjan%27s_strongly_connected_components_algorithm
    """

    index_counter: List[int] = [0]
    stack: List[T] = []
    lowlinks: MutableMapping[T, int] = {}
    index: MutableMapping[T, int] = {}
    result: List[Tuple[T, ...]] = []

    def strongconnect(node: T) -> None:
        # set the depth index for this node to the smallest unused index
        index[node] = index_counter[0]
        lowlinks[node] = index_counter[0]
        index_counter[0] += 1
        stack.append(node)

        # Consider successors of `node`
        try:
            successors = graph[node]
        except KeyError:
            successors = []
        for successor in successors:
            if successor not in lowlinks:
                # Successor has not yet been visited; recurse on it
                strongconnect(successor)
                lowlinks[node] = min(lowlinks[node], lowlinks[successor])
            elif successor in stack:
                # the successor is in the stack and hence in the current
                # strongly connected component (SCC)
                lowlinks[node] = min(lowlinks[node], index[successor])

        # If `node` is a root node, pop the stack and generate an SCC
        if lowlinks[node] == index[node]:
            connected_component = []

            while True:
                successor = stack.pop()
                connected_component.append(successor)
                if successor == node:
                    break
            component = tuple(connected_component)
            # storing the result
            result.append(component)

    for node in graph:
        if node not in lowlinks:
            strongconnect(node)

    return result

==> py_import_cycles-0.3.1.tar.gz: py_import_cycles-0.3.1/py_import_cycles/tarjan.py <==
#!/usr/bin/env python3

from collections.abc import Hashable, Mapping, Sequence
from typing import MutableMapping, Tuple, TypeVar

T = TypeVar("T", bound=Hashable)


def strongly_connected_components(
    graph: Mapping[T, Sequence[T]],
) -> Sequence[Tuple[T, ...]]:
    """
    Tarjan's Algorithm (named for its discoverer, Robert Tarjan) is a graph theory algorithm
    for finding the strongly connected components of a graph.

    Based on: http://en.wikipedia.org/wiki/Tarjan%27s_strongly_connected_components_algorithm
    """

    index_counter: list[int] = [0]
    stack: list[T] = []
    lowlinks: MutableMapping[T, int] = {}
    index: MutableMapping[T, int] = {}
    result: list[Tuple[T, ...]] = []

    def strongconnect(node: T) -> None:
        # set the depth index for this node to the smallest unused index
        index[node] = index_counter[0]
        lowlinks[node] = index_counter[0]
        index_counter[0] += 1
        stack.append(node)

        # Consider successors of `node`
        try:
            successors = graph[node]
        except KeyError:
            successors = []
        for successor in successors:
            if successor not in lowlinks:
                # Successor has not yet been visited; recurse on it
                strongconnect(successor)
                lowlinks[node] = min(lowlinks[node], lowlinks[successor])
            elif successor in stack:
                # the successor is in the stack and hence in the current
                # strongly connected component (SCC)
                lowlinks[node] = min(lowlinks[node], index[successor])

        # If `node` is a root node, pop the stack and generate an SCC
        if lowlinks[node] == index[node]:
            connected_component = []

            while True:
                successor = stack.pop()
                connected_component.append(successor)
                if successor == node:
                    break
            component = tuple(connected_component)
            # storing the result
            result.append(component)

    for node in graph:
        if node not in lowlinks:
            strongconnect(node)

    return result
